/*****************************************************************************
 * File:         biras.cpp                                                   *
 * Version:      4.0                                                         *
 * Description:  Brain Inspired Robot Action Selection (main function)       *
 * Authors:      Tony Prescott & Fernando Montes                             *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

#include <stream.h>
#include <vector>

#define FULL_INTERFACE 1       // set to zero for minimal interface (graph + time_bar)
#define NEW_DA 0               // set to 1 for new da model
#define MOVEMENT 0             // set to zero to disable movement initially

#include "utils/utils.h"       // io, stat, and timer classes and functions
#include "robot/robot.h"       // all perceptual and motor control for the robot
#include "bg/basal_ganglia.h"  // class for the model basal ganglia

#if NEW_DA
#include "bg/bg_extended_f.h"    // *** customisable basal_ganglia method
#else
#include "bg/bg_extended.h"    
#endif

#include "ui/interface.h"      // class for controlling all gui windows
#include "ctrl/controller.h"   // main controller class                        
#include "ctrl/ctrl_motivated.h"   // customisable controller method for adding objects
// #include "ctrl/wall_follower.h"   // customisable controller method for adding objects

// Global variables 

controller      control;
observer_log    observer(0); // set to 1 to switch recording of observer files
interface       iface(FULL_INTERFACE);

// Global functions for Gui callback                       
// the first is the call-back function for handling events, 
// (see p. 27 of the webots reference manual). Events are 
// passed to the interface and then on to the Tools window. 
// The other two methods are called by buttons in the Tools window.

void gui_callback(void)
{
  iface.callback(control.flags());
}

// MAIN 

main()
{
  control.start(&iface, &observer);
  control.run();
}


























