/*****************************************************************************
 * File:         controller_gui.h                                            *
 * Version:      3.0                                                         *
 * Description:  gui_flag and controller_gui classes                         *
 * Authors:      Tony Prescott                                               *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

// defines a class gui_flag which is used to control some aspect of 
// the behaviour of the model online via the tools window
// and defines an abstract base class for the controller which contains
// some basic gui_flags

#ifndef CONTROLLER_GUI
#define CONTROLLER_GUI

class interface;

class gui_flag
{
  bool  aFlag;
  char* aName;
  char* aTrue_label;
  char* aFalse_label;
  int   aId;
  bool  aEnabled_at_start;
 public:
  bool toggle() { 
    // cout << name() << ": toggle from " << (int) aFlag << " to " << !aFlag;
    // cout << " label " << label() << " enabled " << (int) aEnabled <<"\n";
    return aFlag= !aFlag; }
  bool value() { return aFlag; }
  bool reset() { 
    // if (aFlag) cout << name() << ": resetting flag\n"; 
    bool f= aFlag; 
    if (aFlag) aFlag= false; 
    return f; 
  } 
  void set_value(bool v) { aFlag= v; }     // ***
  char* name() { return aName; }
  char* label() { return ((aFlag)? aTrue_label : aFalse_label); }
  int id() { return aId; }
  int set_id(int i) { return aId= i; }
  bool enabled_at_start() { return aEnabled_at_start; }

  gui_flag(bool init, char* nm, char* lt, char* lf, bool enabled= true) :
    aFlag(init), aName(nm), aTrue_label(lt), aFalse_label(lf), aEnabled_at_start(enabled) {}
};

typedef vector <gui_flag*> flag_vector;

class controller_gui
{
 protected:
  interface* aInterface;                   // ptr to user interface 
  gui_flag aTalk;
  gui_flag aMovement;
  gui_flag aEstart;
  flag_vector aFlags;
 
 public:
  flag_vector& flags() { return aFlags; }
  gui_flag* add_flag(gui_flag* f) 
    { aFlags.push_back(f); f->set_id(aFlags.size() - 1); return f; }
  bool talk() { return aTalk.value(); }
  bool movement() { return aMovement.value(); }
  void set_movement(bool b) { aMovement.set_value(b); }
  bool estart() { return aEstart.reset(); }

  bool show_etime(gui_flag&, float, int, int, char*, char);      // defined in interface.h ***
       
  controller_gui::controller_gui() :
    aTalk(false, "Talk", "quiet", "talkative"),
    aMovement(MOVEMENT, "Movement", "---", "run"),
    aEstart(false, "Start experiment", "Start experiment", "Start experiment", true)
    {
      add_flag(&aTalk);
      add_flag(&aMovement);
      add_flag(&aEstart);
    }
};

#endif

















