/*****************************************************************************
 * File:         ctrl/wall_follower.h                                        *
 * Version:      1.0                                                         *
 * Description:  Controller method for wall following robot                  *
 * Authors:      Tony Prescott                                               *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef CTRL_BASIC
#define CTRL_BASIC

void controller::build()
{
  // set the controller name etc.
  set_name("Wallfollow", "1.0", "31st July 2003");

  set_feedback_source(1);              // 0 - GPi, 1 - thalamus
  set_salience_scale(1.0);             // scale all salience values by this amount
  set_salience_inc(0.0);               // increment all salience values by this amount

  // note: computation is ss * (si + sal)
  string fs= (aFeedback_source==1)? "thalamic feedback" : "GPi feedback";
  string sss= "", ssi= "";
  if (salience_scale()!=1.0) sss= " + s_scale (" + ftos(salience_scale(), 1) + ")";
  if (salience_inc()!=0.0) ssi=  " + s_inc (" + ftos(salience_inc(), 1) + ")";
  set_description("Khepera wall following behaviour, " + fs + sss + ssi);

  // set the duration of the experiment in seconds
  experiment_set_duration(30);
  set_record_log(true);
  set_record_file(efolder()+"/ctrl_log.txt");

  // add perceptual sub-systems 
  percept* pwall(add_percept(new wall_ps(aSenses)));		  
  aMonsys= pwall;

  // add behavioural sub-systems with appropriate perceptual/motivational inputs and weights

  // wall seek
  behaviour* bwall(add_behaviour(new wall_seek));
  bwall->add_input(pwall, -0.12);         // wall SHOULD NOT be visible
  bwall->set_threshold(+0.33);            // equivalent to threshold 0.24 + fear of 0.5*0.18

  // nest seek
  behaviour* bnest(add_behaviour(new nest_seek));
  bnest->add_input(pwall, +0.12);         // wall SHOULD be visible
  bnest->set_busy_weight(+0.225);          // busy signal needed as wall may be lost for short periods
  bnest->set_threshold(+0.345);           // equivalent to threshold of 0.24 + fear of 0.5*0.21

  // finish controller set-up
  initialise_arrays();

  // the basal ganglia
  basal_ganglia* bg(new basal_ganglia (n_behaviours(), persistence()));
  add_basal_ganglia(bg);

  bg->set_log_level(0);                      // log file output 0- basic, 1- full
  bg->set_record_log(true);
  bg->set_record_file(efolder()+"/bg_log.txt");

  // print a summary of the model
  print(0);                               // 0 description only, output, 1 basic, 2 detailed, 3 max

  // cerr << "monitoring subsys " << aMonsys->name() << '\n';
  // cout << "experiment duration is " << experiment_duration() << endl;
}

#endif















