/*****************************************************************************
 * File:         motor_plant.h                                               *
 * Version:      3.0                                                         *
 * Description:  motor postprocessing functions                              *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

#include <Khepera.h>
#include <KheperaGripper.h>
#include <stream.h>

class motor_plant {
 private:
  enum Vector_els {eLM, eLP, eRM, eRP, eGM, eGP, eAU, eAM, eAD};
  enum Motor_vals {eUp= 152, eMiddle= 227, eDown= 255, eOpen= 0, eClosed= 1 }; 

  int aLws;
  int aRws;
  int aArm;
  int aGrip;
  int aMax_speed;
  float aMotors[N_MOTOR_SIG];

 public:
  float* motors() { return aMotors; }
  int n_motors() { return N_MOTOR_SIG; }
  void reset() { for (int i=0; i<N_MOTOR_SIG; ++i) aMotors[i]= 0.0; }
  motor_plant() { 
    aLws= aRws= 0; 
    aArm= -1; 
    aGrip= -1; 
    aMax_speed= FORWARD_SPEED;
  }
  void initialise();
  void step(bool);
  void print(); 
};

void motor_plant::step(bool movement)
{
  aArm= aGrip= -1;
  aLws= aRws= 0;
  
  if (movement)
    {
      aLws= (int) ((-aMotors[eLM] + aMotors[eLP]) * aMax_speed);
      aRws= (int) ((-aMotors[eRM] + aMotors[eRP]) * aMax_speed);
      
      if (aMotors[eGM] > 0.0 || aMotors[eGP]> 0.0)
	aGrip= ((-aMotors[eGM] + aMotors[eGP]) >= 0.0)? eClosed : eOpen;

      float sum_a= aMotors[eAD] + aMotors[eAM] + aMotors[eAU];
      if (sum_a > 0.0)
	aArm= (int) ((1.0 / sum_a) *
		     (aMotors[eAD]*eDown + 
		      aMotors[eAM]*(eMiddle - 15) + 
		      aMotors[eAU]*eUp));
      /*
      if (aArm!=-1) 
    	cout << aLws << " " << aRws << " " << aArm << " " << aGrip << endl;
      */
    }

  khepera_set_speed(LEFT, aLws);      
  khepera_set_speed(RIGHT, aRws);
  if (aArm != -1) gripper_set_arm(aArm);          
  if (aGrip != -1) gripper_set_grip(aGrip);
}

void motor_plant::initialise()
{
  khepera_set_speed(LEFT, 0);
  khepera_set_speed(RIGHT, 0);
  gripper_set_arm(eUp);
  gripper_set_grip(eOpen);
}

void motor_plant::print()
{
  for (int i=0; i<N_MOTOR_SIG; ++i) cout << aMotors[i] << " ";
  cout << endl;
}  












