/*****************************************************************************
 * File:         robot.h                                                     *
 * Version:      3.0                                                         *
 * Description:  Definitions for robot interface                             *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Contact:      T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef ROBOT
#define ROBOT

// OFF- 0, ON- 1, defined /usr/local/webots/include/khepera.h 

#define USE_LIGHT ON            // use passive light sensor to distinguish nest 
#define USE_CORNER OFF          // use corner detector to distinguish nest 
#define USE_WALL OFF            // use wall detector to distinguish nest 
#define POSTURE OFF             // if ON then 'postural' control is actively maintained 
                                // e.g. arm position is sent at every step

#define N_IR 8                  // number of infrared sensors on robot 
#define N_MOTORS 4              // number of motors 
#define N_MOTOR_SIG 9           // number of motor signals 

#define USE_RESISTANCE  OFF
#define CAN_RESISTIVITY 25      // can resistivity 

#define OBS_THRESH 30           // Minimum IR proximity value to detect an obstacle        
#define WALL_THRESH 800         // Minimum total IR values to detect a wall, 800               

#define LIGHT_THRESH 100        // maximum value for detecting presence of the light       
#define LIGHT_COUNT 1           // minimum number of 'lit' sensors needed to detect a nest 

#define CORNER_THRESH 4000      // minimum value for detecting presence of a corner        
#define CORNER_OBS 1            // minimum number of obstacles needed to detect a corner   

#define CAN_FRONT 1000          // minimum forward IR activity to detect a can             
#define CAN_SIDE  10            // maximum side IR activity to detect a can                

#define FORWARD_SPEED 15        // default forward speed 

#include "sensory_sys.h"        // class for raw sensory systems
#include "motor_plant.h"        // class for robot motor interface              
                
#include "subsys.h"             // base class for percept & motivation classes  
#include "percept.h"            // generic and specialised percept classes      
#include "motivation.h"         // generic and specialised motivation classes   

#include "behaviour.h"          // base class for behaviours
#include "forage.h"             // specialised behaviour classes for khepera 'foraging'

#endif                          // end of ifndef













































