/*****************************************************************************
 * File:         subsys.h                                                    *
 * Version:      4.0                                                         *
 * Description:  defines a base class of basal ganglia input subsystems      *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef SUBSYS
#define SUBSYS

#include <string.h>

class subsys;
typedef vector <subsys*> svector;

class subsys {
 protected:
  char  *aName;        // subsystem name
  char  *aLabel;       // subsystem label
  float aV;            // current value
  float aVtm1;         // value on the previous time-step
  int   aSwitches;     // no. of times value has changed;
  sensory_sys *aSptr;  // pointer to the raw sensory system
 public:
  float value() { return aV; }
  float valuetm1() { return aVtm1; }
  bool  change() { if (aV==aVtm1) return false; else return true; }
  float set_value(float v) { if (aV!=v) aSwitches++; aVtm1= aV; return aV= v; }
  void  zero(float v= 0.0) { aV= aVtm1 = v; aSwitches= 0; }
  int   switches() { return aSwitches; }
  void  print_analysis_hdr(bool el= false, int verbose= 0) 
    { cout << abb_cstring(aName, 3) << "_sw"; if (el) cout << '\n'; else cout << '\t'; }
  void  print_analysis(bool el= false, int verbose= 0) 
    { cout << aSwitches-1; if (el) cout << '\n'; else cout << '\t'; }
  char* name() { return aName; }
  char* label() { return aLabel; }
  virtual float update()= 0;
  subsys(char*, sensory_sys*, float);
  virtual ~subsys() { delete aName; delete aLabel; }
  virtual void print(int verbose= 1) { if (verbose>=1) cout << name(); }
};

subsys::subsys(char *name, sensory_sys *sp, float init_v= 0.0) 
{
  aName= new char[strlen(name)+1];
  strcpy(aName, name);
  aLabel= new char[3];
  memcpy(aLabel, aName, 2);
  aLabel[3]= 0;
  aSptr= sp;
  aV= init_v;
  set_value(init_v);
}
 

#endif















