/****************************************************************************
* File:         benchmark.cpp                                               *
* Version:      1.0                                                         *
* Description:  basal ganglia test/benchmark program                        *
* Authors:      Tony Prescott                                               *
* Contact:      T.j.prescott@shef.ac.uk                                     *
*****************************************************************************/

/* runs a five-channel model to convergence with 0, 1, then 2 active channels */
/* results are printed to the screen or to a log-file                         */

#define SAVE 0     // if 1 the write to file
#define LEVEL 0    // 0 record data just after each cycle, 1 record on each step
#define STEP 0     // 0 -run till converged, 1 -run STEP steps
#define VALUE 2    // 0-net input, 1-activation, 2-output
#define EXTENDED 1

#include "../bg/basal_ganglia.h"

#if EXTENDED
#include "../bg/bg_extended.h"
#else
#include "../bg/bg_basic.h"
#endif

int main()
{
  using namespace bgs;
  
  const int n_epochs= 3;
  const int active_channels= 2;
  const char* vnames[]= {"net input", "activation", "output"};
  float saliences[3][5]= {
    { 0.0, 0.0, 0.0, 0.0, 0.0 },       // ***
    { 0.6, 0.0, 0.0, 0.0, 0.0 },
    { 0.3, 0.6, 0.0, 0.0, 0.0 }
  };
 

  basal_ganglia* bg= new basal_ganglia(5, NULL, active_channels);
  bg->set_record_log(SAVE);
  bg->set_record_level(LEVEL);
  bg->set_record_file("log.txt");

  bg->print(0);
  if (SAVE) {
    bg->print(3, "model.txt");
    bg->start_log();
  }

  
  for (int e=0; e<n_epochs; ++e)
    {
      float* salience= saliences[e];
      cout << endl;
      bg->print_array("Sal", salience);
      
      if (STEP==0)
	{
	  bg->converge(salience, e, true);
	  if (!SAVE) bg->print_values((Leaky_value) 2);
	}
      else
	{
	  for (int i=0; i<STEP; ++i) {
	    bg->step(salience);
	    bg->log(i);
	  }
	  cout << vnames[VALUE] << " after " << STEP << " steps:\n";
	  if (!SAVE) bg->print_values((Leaky_value) VALUE);
	}
      // cout << "Max delta= " << bg->maxd() << "\n\n";
    }
  if (SAVE) {
  	bg->finish_log();
    cout << "finished, results are in log.txt\n";
    }
  return 1;
}

















