/* 
   test_bg/mult.cpp
   Basal ganglia test/benchmark program for multi-channel (5) salience inputs.
   Author: Tony Prescott
   Contact: t.j.prescott@shef.ac.uk
*/

#define SAVE 1  // if 1 the write to file
#define LEVEL 0 // 0 record data just after each cycle, 1 record on each step
#define BG_STEP 0  // 0 -run till converged, otherwise -run BG_STEP steps
#define VALUE 2 // 0-net input, 1-activation, 2-output
#define STEPS 100
#define EPOCHS 2
#define MAX_STEPS 400
#define CHANNELS 5
#define ACT_CHAN 5

#include "../bg/basal_ganglia.h"

#define EXTENDED 1
#define DOPAMINE 0.0         // set to -1.0 to use default value

#if EXTENDED
#include "../bg/bg_extended.h"
#else
#include "../bg/bg_basic.h"
#endif

#include <vector>

int main()
{
  using namespace bgs;
  
  const char* vnames[]= {"net input", "activation", "output"};
  float saliences[MAX_STEPS][CHANNELS];
  float increment= 1.00/(STEPS - 1);

  for (int e=0; e<EPOCHS; ++e)
    {
      for (int i=0; i<STEPS; ++i)
	{
	  int k= e*STEPS + i;
	  if (e==0)
	    {
	      saliences[k][0]= 0.7;
	      saliences[k][1]= 1.3;
	      saliences[k][2]= 0.3;
	      saliences[k][3]= -0.1;
	      saliences[k][4]= -0.5;
	    }
	  else if (e==1)
	    {
	      saliences[k][0]= 0.3;
	      saliences[k][1]= 0.9;
	      saliences[k][2]= 0.7;
	      saliences[k][3]= 0.3;
	      saliences[k][4]= 0.3;
	    }
	  else if (e==2)
	    {
	      saliences[k][0]= 0;
	      saliences[k][1]= 1 - i*increment;
	    }
	  else if (e==3)
	    {
	      saliences[k][0]= i*increment;
	      saliences[k][1]= i*increment*0.5;
	    }
	  if (ACT_CHAN < CHANNELS)
	    for (int j= ACT_CHAN; j<CHANNELS; ++j) saliences[k][j]= 0.0;
	}
    }

  basal_ganglia* bg= new basal_ganglia(CHANNELS, NULL, ACT_CHAN);
    
  if (DOPAMINE>=0.0) bg->set_dopamine(DOPAMINE);
  bg->set_log_level(1);                          // log file output 0- basic, 1- full

  bg->set_record_log(SAVE);
  bg->set_record_level(LEVEL);
  bg->set_record_file("log.txt");

  bg->print(1);
  if (SAVE) {
    bg->print(3, "model.txt");
    bg->start_log();
  }

  for (int ts=0; ts<EPOCHS*STEPS; ++ts)
    {
      float* salience= saliences[ts];
      //  cout << endl;
      // bg->print_array("Sal", salience);
      
      if (BG_STEP==0)
	{
	  bg->converge(salience, ts, false);             // true to print convergence details
	  if (!SAVE) bg->print_values((Leaky_value) 2);
	}
      else
	{
	  for (int i=0; i<BG_STEP; ++i) {
	    bg->step(salience);
	    bg->log(i);
	  }
	  cout << vnames[VALUE] << " after " << BG_STEP << " steps:\n";
	  if (!SAVE) bg->print_values((Leaky_value) VALUE);
	}
      // cout << "Max delta= " << bg->maxd() << "\n\n";
    }
  if (SAVE) bg->finish_log();
  cout << "finished, results are in log.txt\n";
  return 1;
}

















