/* 
   test_bg/step.cpp
   Basal ganglia test/benchmark program for step change salience inputs.
   Author: Tony Prescott
   Contact: t.j.prescott@shef.ac.uk
   
   does exhaustive testing of a 2-d salience space with granularity INCREMENT
*/

#define SAVE 1           // if 1 the write to file
#define LEVEL 0          // 0 record data just after each cycle, 1 record on each step
#define STEP 0           // 0 -run till converged, 1 -run STEP steps
#define VALUE 2          // 0-net input, 1-activation, 2-output
#define MAX_EPOCHS 10000

#include "../bg/basal_ganglia.h"

#define EXTENDED 1
#define DOPAMINE 1     // set to 1 for to get DA value from user otherwise use default
#define HYSTERESIS 1   // set to 1 to get y/n from user otherwise off

#if EXTENDED
#include "../bg/bg_extended.h"
#else
#include "../bg/bg_basic.h"
#endif


int main()
{
  using namespace bgs;
  
  const float increment= 0.1;
  const int active_channels= 2;
  const char* vnames[]= {"net input", "activation", "output"};

  int n_vals= (int) (1/increment) + 1;
  int n_epochs= n_vals * n_vals;
  float saliences[MAX_EPOCHS][5];

  for (int i= 0; i<n_vals; ++i)
    {
      for (int j= 0; j<n_vals; ++j)
	{
	  int e= (i * n_vals) + j;
	  saliences[e][0]= i*increment;
	  saliences[e][1]= j*increment;
	  for (int k= 2; k<5; ++k) saliences[e][k]= 0.0;
	}
    }
  
  basal_ganglia* bg= new basal_ganglia(5, NULL, active_channels);

  if (DOPAMINE) bg->set_dopamine(get_float("dopamine? "));

  bool hysteresis= false;         // if false then primes bg with zero input before each epoch
                                  // otherwise bg begins in the same state as the end of previous epoch
  if (HYSTERESIS) hysteresis= get_y_or_n("hysteresis? ");

  if (hysteresis) cout << "hysteresis enabled\n";

  bg->set_log_level(0);                          // log file output 0- basic, 1- full

  bg->set_record_log(SAVE);
  bg->set_record_level(LEVEL);
  bg->set_record_file("log.txt");

  bg->print(0);
  if (SAVE) {
    bg->print(3, "model.txt");
    bg->start_log();
  }

  for (int e=0; e<n_epochs; ++e)
    {
      float* salience= saliences[e];
      // cout << endl;
      // bg->print_array("Sal", salience);
      
      if (!hysteresis) bg->calibrate();
      if (STEP==0)
	{
	  bg->converge(salience, e, false);
	  if (!SAVE) bg->print_values((Leaky_value) 2);
	}
      else
	{
	  for (int i=0; i<STEP; ++i) {
	    bg->step(salience);
	    bg->log(i);
	  }
	  cout << vnames[VALUE] << " after " << STEP << " steps:\n";
	  if (!SAVE) bg->print_values((Leaky_value) VALUE);
	}
      // cout << "Max delta= " << bg->maxd() << "\n\n";
    }
  if (SAVE) bg->finish_log();
  cout << endl;
  cout << "finished, results are in log.txt\n";
  return 1;
}

















