/* 
   test_bg/selection.cpp
   Basal ganglia test/benchmark program for gradual change with hysteresis
   Author: Tony Prescott
   Contact: t.j.prescott@shef.ac.uk
*/

#define SAVE 1           // if 1 then write model to file
#define LEVEL 0          // 0 record data just after each cycle, 1 record on each step
#define STEP 0           // 0 -run till converged, 1 -run STEP steps
#define VOLATILE 0       // 1 if losing channels 'track' winners

#include "../bg/basal_ganglia.h"
#include <math.h>

#define CONTOUR 1      // 1- Selection (Contour), 1- Efficiency (c), 2- Distortion (c), 3- Winner 
                       // 0- C2win
#define EXTENDED 1
#define NEW_DA 0
#define DOPAMINE 1     // set to 1 for to get DA value from user otherwise use default
#define HYSTERESIS 1

#if EXTENDED
#include "../bg/bg_extended.h"
#else
#include "../bg/bg_basic.h"
#endif

const float inactive= 0.0;

#define P_DA
#ifdef P_DA
const int   n_params= 1;
const char* abb_name= "DA";
const char* par_name[n_params]= {"Dopamine"}; 
const float par_range[n_params][3]= {{0.2, 0.3, 0.01}};
// const float par_range[n_params][3]= {{0.18, 0.22, 0.02}};
const int   aff_n= -1;
#endif

//#define P_DA2
#ifdef P_DA2
const int   n_params= 1;
const char* abb_name= "DA";
const char* par_name[n_params]= {"Dopamine"}; 
const float par_range[n_params][3]= {{0.18, 0.46, 0.01}};
const int   aff_n= -1;
#endif

// #define P_MOT
#ifdef P_MOT
const int   n_params= 1;
const char* abb_name= "mot";
const char* par_name[n_params]= {"mot_thl"}; 
const float par_range[n_params][3]= { {0.2, 0.6, 0.02}};
const int   aff_n= 0;
#endif

// #define P_THL
#ifdef P_THL
const int   n_params= 1;
const char* abb_name= "thl";
const char* par_name[n_params]= {"gpi_thl" }; 
const float par_range[n_params][3]= { {-1.0, 0.0, 0.1}};
const int   aff_n= 1;
#endif

// #define P_TRN
#ifdef P_TRN
const int   n_params= 2;
const char* abb_name= "trn";
const char* par_name[n_params]= {"trn_thl_w", "trn_thl_b" }; 
const float par_range[n_params][3]= { {-0.15, 0.0, 0.05}, {-0.8, 0.0, 0.1}};
const int   aff_n= 2;
#endif

const int active_channels= 2;
const int total_channels= 5;
const int n_results= 7;
const int n_stats= 16;
const char* vnames[]= {"net input", "activation", "output"};

float stat_array[n_stats];      // params * stats

int n_pvs= 122; 
int n_pvs1= 0;

ofstream outs;


void set_n_pvs()
{
  if (n_params==1)
    n_pvs= (int) ((par_range[0][1] - par_range[0][0])/par_range[0][2] + 1.5);
  else
    {
      n_pvs1= (int) ((par_range[1][1] - par_range[1][0])/par_range[1][2] + 1.5);
      n_pvs= n_pvs1* ((int) ((par_range[0][1] - par_range[0][0])/par_range[0][2] + 1.5));
    }
  // cout << "n_pvs " << n_pvs << ", n_pvs1 " << n_pvs1 << endl;
}
	
float get_par(int i, int n= 0)
{
	if (i > n_pvs) return -1;
	else return par_range[n][0] + i * par_range[n][2];
}	
  
void write_par(int i, int c1, int c2)
{
  if (n_params==1) cout << get_par(i);
  else cout << get_par(c1, 0) << "|" << get_par(c2, 1);
}

int wflag;  // note: global variable!!

int get_flags(bgs::basal_ganglia *bg, int flags[], float sa[])
{
  int sf= bg->sflag();
  int win= bg->winner();
  int n_draw= bg->draw();
  float ga[total_channels];
  bg->gates(ga);
  // sflag: none= 0, part= 1, clean= 2, intf= 3, dual= 4, mult= 5

  sa[0]++;
  flags[0]= (sf<3)? (int) sf : ((int) sf)+1;
  if (sf==2 && win==0) flags[0]= 3;
  
  if (sf==0)                              { flags[0]= 0; flags[1]= 0; sa[1]++; }
  else if (sf==1 && win==1 && ga[0]<0.05) { flags[0]= 1; flags[1]= 1; sa[2]++; }
  else if (sf==1 && ga[1]<0.05)           { flags[0]= 1; flags[1]= 4; sa[3]++; }
  else if (sf==1 && win==1)               { flags[0]= 1; flags[1]= 1; sa[4]++; }
  else if (sf==1)                         { flags[0]= 1; flags[1]= 4; sa[5]++; }
  else if (sf==2 && win==1)               { flags[0]= 2; flags[1]= 2; sa[6]++; }
  else if (sf==2)                         { flags[0]= 3; flags[1]= 5; sa[7]++; }
  else if (sf==3 && win==1)               { flags[0]= 4; flags[1]= 3; sa[8]++; }
  else if (sf==3)                         { flags[0]= 4; flags[1]= 6; sa[9]++; }
  else if (sf>=4 && win==1)               { flags[0]= 5; flags[1]= 7; sa[10]++; }
  else if (sf>=4)                         { flags[0]= 5; flags[1]= 7; sa[11]++; }
  else cout << "This shouldn't happen\n";
  
  if (n_draw==0 && win==1) { sa[12]++; wflag= 2; }     // channel 2 wins
  else if (n_draw==0)      { sa[13]++; wflag= 1; }     // another channel wins
  else if (ga[win]>=0.01)  { sa[14]++; wflag= 3; }     // tie
  else                     { sa[15]++; wflag= 0; }     // no selection

  return flags[0];
}

int contour_hdr(string fname)
{
  if (redirect_cout(outs, fname, false)) 
    {
      cout << "sal1" << '\t' << "sal2" << '\t';
      for (int i=0; i<n_pvs; ++i) 
	if (n_params==1) cout << abb_name << get_par(i) << '\t';
	else cout << get_par(i,0) << "|" << get_par(i, 1) << '\t';
      cout << endl;
      reset_format_flags();
      reinstate_cout(outs);
      return 1;
    }
  else return 0;
}

void print_contour(string fname, float* da, int n_epochs, double inc)
{
  if (redirect_cout(outs, fname, true)) 
    {
      table_format_flags(3, false);
      for (int e1= 0; e1< n_epochs; ++e1)
	for (int e2= 0; e2< n_epochs; ++e2)
	  {
	    cout << e1*inc << '\t' << e2*inc << '\t';
	    for (int ind= 0; ind<n_pvs; ++ind)
	      {
		cout << da[e1*n_epochs*n_pvs+e2*n_pvs+ind];
		if (ind+1<n_pvs) cout << '\t';
	      }
	    cout << endl;
	  }
      reset_format_flags();
      reinstate_cout(outs);
    }
}

void print_pc(bool hdr, char* nm, int a, int b= -1, int c= -1, int d= -1)
{
  if (hdr) cout << nm << '\t';
  else
  {
    float sum= stat_array[a];
    if (b>0) sum+= stat_array[b];
    if (c>0) sum+= stat_array[c];
    if (d>0) sum+= stat_array[d];
    cout << (100.0 * sum / stat_array[0]) << '\t';
  }
}

void print_pcs(bool hdr)
{  	
  cout << '\t';
  print_pc(hdr, "no_sel_", 1);
  print_pc(hdr, "prt_tot", 2, 3, 4, 5);
  print_pc(hdr, "cln_tot", 6, 7);
  print_pc(hdr, "itf_tot", 8, 9);
  print_pc(hdr, "dl_tot_", 10, 11);
  print_pc(hdr, "prt_one", 2, 3);
  print_pc(hdr, "prt_bth", 4, 5);
  print_pc(hdr, "2_clean", 6);
  print_pc(hdr, "1_clean", 7);
  print_pc(hdr, "2pt_tot", 2, 4, 5, 9);
  print_pc(hdr, "1pt_tot", 3, 4, 5, 8);
  print_pc(hdr, "2cl_tot", 6, 8, 10, 11);
  print_pc(hdr, "1cl_tot", 7, 9, 10, 11);
  print_pc(hdr, "2_wins_", 12);
  print_pc(hdr, "1_wins_", 13);
  print_pc(hdr, "tie____", 14);
  print_pc(hdr, "no_sele", 15);
  // print_pc(hdr, "2_part_", 2);
  // print_pc(hdr, "1_part_", 3);
  // print_pc(hdr, "2>=1prt", 4);
  // print_pc(hdr, "1>2part", 5);
  // print_pc(hdr, "2cl_1pt", 8);
  // print_pc(hdr, "1cl_2pt", 9);
  // print_pc(hdr, "2>=1dl_", 10);
  // print_pc(hdr, "1>2dual", 11);
  cout << endl;
}

int stats_hdr()
{
  if (redirect_cout(outs, "contours/stats.txt", false)) {
    cout << "selection statistics for ";
    if (n_params==1) cout << par_name[0] << "\n\n";
    else cout << par_name[0] << ", " << par_name[1] << "\n\n";
  	cout << abb_name;
    print_pcs(true);
    reinstate_cout(outs);
    return 1;
  }
  else return 0;
}

int stats2_hdr(bgs::basal_ganglia *bg)
{
  if (redirect_cout(outs, "contours/stats2.txt", false)) 
  {
    if (aff_n>=0) cout << abb_name << '\t';
	bg->print_analysis_hdr(true);
    reinstate_cout(outs);
 	return 1;
  }
  return 0;
}

int contour(bgs::basal_ganglia *bg)
{
  if (!stats_hdr()) return 0;
  if (!stats2_hdr(bg)) return 0;
  if (!contour_hdr("contours/select.txt")) return 0;
  if (!contour_hdr("contours/winner.txt")) return 0;
  if (!contour_hdr("contours/efficiency.txt")) return 0;
  if (!contour_hdr("contours/distortion.txt")) return 0;
  
  double increment= get_float("Salience increment? ");
  int n_epochs= (int) (1.0/increment + 1.5);
  
  cout << "Generating " << n_pvs << " x " << n_epochs << " x " << n_epochs;
  cout << " = " << n_pvs*n_epochs*n_epochs << " data points\n";
  
  float salience[total_channels];
  int dsz= n_epochs*n_epochs*n_pvs;
  float* data= new float[dsz];
  float* data_w= new float[dsz];
  float* data_e= new float[dsz];
  float* data_d= new float[dsz];

  int   flags[2];
  cout << "n_epochs " << n_epochs << " max sal " << increment*(n_epochs-1) << endl;
  
  int  c1= 0, c2= 0;
  for (int ind= 0; ind<n_pvs; ++ind)
    {
      float v, v1;
      
      if (n_params==1) v= get_par(ind);
      else
	{
	  v= get_par(c1, 0);
	  v1= get_par(c2, 1);
	}
     
      if (aff_n<0) bg->set_dopamine(v);
      else if (aff_n==0) bg->get_afferent(0)->set_weight_w(v);
      else
	{
	  bg->get_afferent(1)->set_weight_w(v);
	  bg->get_afferent(1)->set_weight_b(v1);
	}
      bg->zero();
      
      for (int i= 0; i<n_stats; ++i) stat_array[i]= 0;
      cout << par_name[0] << " " << v;
      if (n_params==2) cout << " " << par_name[1] << " " << v1;
      cout << " getting " << n_epochs*n_epochs << " data points\n";
      flush(cout);
      
      for (int e1= 0; e1<n_epochs; ++e1)
    	{
	  salience[0]= e1*increment; 
	  for (int i=1; i<total_channels; ++i) salience[i]= inactive;
	  if (VOLATILE) salience[2]= ((salience[0] - 0.2)<0.0)? 0.0 : (salience[0] - 0.2);
	  bg->calibrate(false);
	  
	  for (int e2= 0; e2<n_epochs; ++e2)
	    {
	      if (!HYSTERESIS) bg->calibrate(false);
	      salience[1]= e2*increment;
	      if (VOLATILE) salience[3]= ((salience[1] - 0.2)<0.0)? 0.0 : (salience[1] - 0.2);
	      bg->converge(salience, e1*n_epochs+e2, false);
	      int di= e1*n_epochs*n_pvs+e2*n_pvs+ind;
	      data[di]=  get_flags(bg, flags, stat_array);
	      data_w[di]= wflag;
	      data_e[di]= bg->efficiency();
	      data_d[di]= bg->distortion();
	    }
	}
      
      if (redirect_cout(outs, "contours/stats.txt", true))
	{
	  table_format_flags(3, false);
	  write_par(ind, c1, c2);
	  print_pcs(false);
	  reset_format_flags();
	  reinstate_cout(outs);
	}
      
      if (redirect_cout(outs, "contours/stats2.txt", true)) 
	{
	  table_format_flags(3, false);
	  if (aff_n>=0) 
	    { write_par(ind, c1, c2); cout << '\t'; }
	  bg->print_analysis_r(true);
	  reset_format_flags();
	  reinstate_cout(outs);
	}
 
      if (n_params!=1)
	{
	  c2++;
	  if (c2==n_pvs1) { c1++; c2= 0; }
	}
    }
  // cout << "increment is " << increment << endl;
  print_contour("contours/select.txt", data, n_epochs, increment);
  print_contour("contours/winner.txt", data_w, n_epochs, increment);
  print_contour("contours/efficiency.txt", data_e, n_epochs, increment);
  print_contour("contours/distortion.txt", data_d, n_epochs, increment);
  return 0;
}

int C2win(bgs::basal_ganglia *bg)
{
  double increment= get_float("C1 increment? ");
  double c2increment= get_float("C2 increment? ");
  int n_epochs= (int) (1.0/increment + 1.5);
  
  bool row_wise= false;
  if (redirect_cout(outs, "C2win.txt", false)) 
    {
      if (row_wise)
	{
	  cout << "sal1" << '\t';
	  for (int i=0; i<n_pvs; ++i) 
	    if (n_params==1) cout << abb_name << get_par(i) << '\t';
	    else cout << get_par(i,0) << "|" << get_par(i, 1) << '\t';
	  cout << endl;
	}
      else
	{
	  cout << abb_name << '\t';
	  for (int i=0; i<(n_epochs-1); ++i)
	    cout << "c1_" << i*increment << '\t';
	  cout << "c1_" << (n_epochs-1)*increment << '\n';
	}
      reinstate_cout(outs);
    }
  else return 0;
  
  cout << "Generating " << n_pvs << " x " << n_epochs;
  cout << " = " << n_pvs*n_epochs << " data points\n";
  
  float salience[total_channels];
  float* data= new float[n_epochs*n_pvs];
  int   flags[2];
  // cout << "n_epochs " << n_epochs << " max sal " << increment*(n_epochs-1) << endl;
  
  int  c1= 0, c2= 0;
  for (int ind= 0; ind<n_pvs; ++ind)
    {
      float v, v1;
      
      if (n_params==1) v= get_par(ind);
      else
	{
	  v= get_par(c1, 0);
	  v1= get_par(c2, 1);
	  c2++;
	  if (c2==n_pvs1) { c1++; c2= 0; }
	}
      
      if (aff_n<0) bg->set_dopamine(v);
      else if (aff_n==0) bg->get_afferent(0)->set_weight_w(v);
      else
	{
	  bg->get_afferent(1)->set_weight_w(v);
	  bg->get_afferent(1)->set_weight_b(v1);
	}
      bg->zero();
      
      cout << par_name[0] << " " << v;
      if (n_params==2) cout << " " << par_name[1] << " " << v1;
      cout << " getting " << n_epochs << " data points\n";
      flush(cout);
      
      for (int e1= 0; e1<n_epochs; ++e1)
    	{
	  salience[0]= e1*increment; 
	  for (int i=1; i<total_channels; ++i) salience[i]= inactive;
	  if (VOLATILE) salience[2]= ((salience[0] - 0.2)<0.0)? 0.0 : (salience[0] - 0.2);
	  bg->calibrate(false);
	
	  int flag= -1, e2= 0;
	  salience[1]= (salience[0]<0.05)? 0.0 : (salience[0] - 0.05);

	  while (flag<1 && salience[1]<1.01)
	    {
	      salience[1]+=c2increment;
	      if (VOLATILE) salience[3]= ((salience[1] - 0.2)<0.0)? 0.0 : (salience[1] - 0.2);
	      bg->converge(salience, e1*n_epochs+e2, false);
	      float ga[total_channels];
	      bg->gates(ga);
	      // cerr << salience[0] << " " << salience[1] << " " << ga[0] << " " << ga[1] << endl;
	      if (ga[1]==ga[0]) flag=0;
	      if (ga[1]>ga[0]) flag+=2;
	      e2++;
	    }
	      data[e1*n_pvs+ind]=  (flag==1)? salience[1] : 999;
	}  
    }

  if (redirect_cout(outs, "C2win.txt", true)) 
    {
      table_format_flags(3, false);
      if (row_wise)
	{
	  for (int e1= 0; e1< n_epochs; ++e1)
	    {
	      cout << e1*increment << '\t';
	      for (int ind= 0; ind<n_pvs; ++ind)
		{
		  if (data[e1*n_pvs+ind]<=1.0) cout << data[e1*n_pvs+ind];
		  if (ind+1<n_pvs) cout << '\t';
		}
	      cout << endl;
	    }
	}
      else
	{
	  for (int ind= 0; ind<n_pvs; ++ind)
	    {
	      cout << get_par(ind) << '\t';
	      for (int e1= 0; e1< n_epochs; ++e1)
		{
		  if (data[e1*n_pvs+ind]<=1.0) cout << data[e1*n_pvs+ind];
		  if ((e1+1)<n_epochs) cout << '\t';
		}
	      cout << endl;
	    }
	}
      reset_format_flags();
      reinstate_cout(outs);
    }
  return 1;
}

int main()
{	
  using namespace bgs;
  
  set_n_pvs();
  if (n_pvs>121){ cerr << "Too many params\n"; return 0; }
  
  basal_ganglia* bg= new basal_ganglia(5, NULL, active_channels);
    
  bg->print(0);
  // bg->print_afferents();

  for (int i=0; i<n_params; ++i)
  {
  	cout << "\nInvestigating parameter " << par_name[i];
  	cout << " from " << par_range[i][0] << " to " << par_range[i][1];
  	cout << " in increments of " << par_range[i][2] << "\n";
  }
  cout << "total increments is " << n_pvs << "\n\n";

  if (CONTOUR==1) cout << "Contours show: selection outcome, winner, efficiency, and distortion\n";
  else cout << "Data show C2 salience required to win against a given C1 salience\n";
  bg->set_log_level(0);                          // log file output 0- basic, 1- full
  bg->set_record_log(false);
  bg->set_record_level(LEVEL);
  bg->set_record_file("contours/log.txt");

  if (SAVE) {
    bg->print(3, "contours/model.txt");
    bg->start_log();
  }

  int flag;
  if (CONTOUR==1) flag= contour(bg);
  else flag= C2win(bg);
  cout << endl;
  cout << "finished\n";
  return 1;
}

















