/*****************************************************************************
 * File:         ethogram_window.h                                           *
 * Version:      4.0                                                         *
 * Description:  window class for displaying behavioural activity            *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef ETHOGRAM_WINDOW
#define ETHOGRAM_WINDOW

#include "iwindow.h"

class ethogram_window : public iwindow
{
 private:
  int           aWx;
  int           aWy;
  int           aWw;
  int           aWh;
  int           aFw;                   // frame size
  int           aFh;
  int           aFx;                   // frame offsets
  int           aFy;
  int           aRowh;                 // row height
  gui_line draw_behaviour(int, int, float, int);
 public:
  void remake(bvector& behaviours);
  void create(bvector&);
  void update(bvector&);
  virtual char* name() { return "ethogram"; } 
  ethogram_window() : iwindow() {}
};


void ethogram_window::remake(bvector& behaviours)
{
  if (aExists)
    {
      // cout << "remaking ethogram window\n";
      gui_window_delete(aWindow);
      aWindow= NULL;
      aExists= 0;

      create(behaviours);
    }
}

void ethogram_window::create(bvector& behaviours)
{
  const int lx_offset= 5;
  int nbv, ly_offset;

  if (!aExists)
    {
      nbv= (int) behaviours.size();

      aWx= 0; aWy= 227;             // x, y, position
      aWw= 410; aWh= 200;           // width, height
      aFx= 30; int aFy= 20;         // x, y border 
      aFw= aWw - (2*aFx);           // drawing frame width
      aFh= aWh - (2*aFy);           //    "      "   height      
      aRowh= (int) (aFh / (nbv+1)); // row height
      
      ly_offset= (int) (aRowh / 2); // label y offset

      aWindow= gui_window_new(name(), aWx, aWy, aWw, aWh); 
      
      // frame
      gui_line_new(aWindow, aFx, aFy+aFh, aFx+aFw, aFy+aFh, GUI_BLACK);
      gui_line_new(aWindow, aFx, aFy, aFx, aFy+aFh, GUI_BLACK);

      // labels
      
      gui_label_new(aWindow, lx_offset, aFy + ly_offset, "Nu");
      for (int i=0; i<nbv; i++)
	gui_label_new(aWindow, lx_offset, aFy + ly_offset + (i+1)*aRowh, 
		      behaviours[i]->label());
      aWindow_frame= gui_rectangle_new(aWindow, aFx+1, aFy, aFw, aFh, GUI_GREY, true);
      
      gui_window_show(aWindow);
      aExists= 1;
      aWindow_clock= 0;
    }
}

gui_line ethogram_window::draw_behaviour(int time, int row, float val, int bn= -1)
{
  const float thresh= 0.05;  // *** minimum activity needed to flag an active channel
  const int ex_offset= 31, ey_offset=25;
  int x, y1, y2, height;

  uint32 c= get_behav_color(bn);

  height= (int) (aRowh * 0.7);
  x= ex_offset + time;
  y1= ey_offset + (row * aRowh);
  y2= y1 + height;

  if (val > thresh)
    return gui_line_new(aWindow, x, y1 + (int) ((1.0 - val) * height), x, y2, c);  
  else
    return gui_line_new(aWindow, x, y1, x, y2, GUI_GREY);

}

void ethogram_window::update(bvector& behaviours)
{
  if (aExists)
    {
      if (aWindow_clock % (aFw - 5)  == 0) remake(behaviours);  // clear();
      // output has reach end of frame      

      float g, max_gate= 0.0;

      for (uint i= 0; i<behaviours.size(); i++) // ***
	{
	  // draw_behaviour(aWindow_clock, i, behaviours[i]->active());
	  g= behaviours[i]->gate();
	  draw_behaviour(aWindow_clock, i+1, g, i);
	  if (g>max_gate) max_gate= g;
	}
      if (max_gate<1.0)
	draw_behaviour(aWindow_clock, 0, (1.0 - max_gate), -1);
      aWindow_clock++;
    }
}

#endif





