/*****************************************************************************
 * File:         observer_log.h                                              *
 * Version:      3.0                                                         *
 * Description:  class and methods for generating Observer Pro data files    *
 * References:                                                               *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Contact:      T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#include <iostream>
#include <fstream>
#include <vector>
#include <stdio.h>
#include "../utils/utils.h"

#define MAX_STREAMS 10

// typedef vector <ofstream> ovector;

class observer_log
{
 private:
  ofstream aObs_streams[MAX_STREAMS];

  int aNof;
  int aNoacttm1;
  int aRecord_obs;
  int aStart_obs;
 
  void open_streams(bvector&);
  void close_streams();
  void write_header(ofstream&);
  void write_footer(ofstream&, double); 
 public: 
  void start(bvector&);
  void step(bvector&, double);
  void finish(double);
  observer_log(int);
};

observer_log::observer_log(int record)
{
  aNoacttm1= OFF;
  aRecord_obs= record;
  aStart_obs= OFF;
}

/*============================ PRIVATE METHODS ==============================*/

void observer_log::open_streams(bvector& behaviours)
{ 
  char *obsdir= "Observer/";
  char s[80];
  int nbv= behaviours.size();
  
  aNof= nbv+1;
  // aObs_streams.resize(nbv+1);         // allocates the correct number of output streams

  for (int i= 0; i<nbv; i++) 
    {
      strcpy(s, obsdir);
      strcat(s, behaviours[i]->name());
      strcat(s, ".odf");
      aObs_streams[i].open(s);
      i++;
    }
  aObs_streams[nbv].open("none.odf");
}

void observer_log::close_streams()
{
  for (int i=0; i<aNof; i++)
    {
      if (aObs_streams[i].is_open())
	{
	  aObs_streams[i].flush();
	  aObs_streams[i].close();
	}
    }
}

void observer_log::write_header(ofstream& disp) 
{
  if (disp.is_open())
    {
      osprint(disp, "FORAG.CNF\n");
      osprint(disp, "First observation\n");
      osprint(disp, "00-00-0000\n");
      osprint(disp, "00:00:00.00\n");
      osprint(disp, "{media}\n");
      osprint(disp, "c:mediafilename\n");
      osprint(disp, "{indvar}\n");
      osprint(disp, "0.5\n");
      osprint(disp, "1\n");
      osprint(disp, "{start}\n");
      osprint(disp, "0.000 off\n");
      disp.flush();
    }
  else cout << "warning obs file not open\n";
}

void observer_log::write_footer(ofstream& disp, double t) 
{
  char s[80];

  if (disp.is_open())
    {
      sprintf(s, "%3.3f {end}\n", t);
      osprint(disp, s);
      osprint(disp, "{notes}\n");
    }
}


/*============================= PUBLIC METHODS ==============================*/

void observer_log::start(bvector& behaviours)
{
  if (aRecord_obs == ON && aStart_obs == OFF) 
    {   
      cout << "Starting Observer records\n";    
      open_streams(behaviours);
      for (int i=0; i<aNof; i++) write_header(aObs_streams[i]);
      for (uint i=0; i<behaviours.size(); i++)
	behaviours[i]->write_observer(aObs_streams[i], 0.0, ON);
      aStart_obs= ON;
    }
}

void observer_log::step(bvector& behaviours, double t)
{
  int noact= ON;
  int nbv= behaviours.size();
  char s[80];

  if (aRecord_obs == ON && aStart_obs == ON)
    {
      
      for (int i= 0; i<nbv; i++)
	{
	  behaviours[i]->write_observer(aObs_streams[i], t);
	  if (behaviours[i]->active() == ON) noact= OFF;
	}
      
      if ((noact == ON) && aNoacttm1 == OFF)    // write the entry for the 'none' behaviour   
	{ 
	  sprintf(s, "%3.3f %s\n", t, "none"); 
	  osprint(aObs_streams[nbv], s); 
	  aNoacttm1 = ON;       
	}
      else if (noact == OFF && aNoacttm1 == ON)
	{
	  sprintf(s, "%3.3f off\n", t); 
	  osprint(aObs_streams[nbv], s); 
	  aNoacttm1 = OFF;  
	}
    }
}

void observer_log::finish(double t)
{
  if (aRecord_obs == ON && aStart_obs == ON) 
    {
      printf("Closing Observer records");
      for (int i=0; i<aNof; i++)
	write_footer(aObs_streams[i], t);
      close_streams();
      aRecord_obs= OFF;
      aStart_obs= OFF; 
    }
}











