/*****************************************************************************
 * File:         percepts_window.h                                           *
 * Version:      4.0                                                         *
 * Description:  window class for displaying perceptual variables            *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef PERCEPTS_WINDOW
#define PERCEPTS_WINDOW

#include "iwindow.h"

class percepts_window : public iwindow
{
 private:
  int           aWx;
  int           aWy;
  int           aWw;
  int           aWh;
  int           aFx;
  int           aFy;
  int aN_percepts;
  gui_rectangle *aCell_rects;
  gui_label *aCell_labels;
 public:
  void remake(pvector&);
  void create(pvector&);
  void update(pvector&);
  virtual char* name() { return "percepts"; }
  percepts_window(int wx= 0, int wy= 676, int ww= 410, int wh= 60, int fx= 30, int fy= 10) 
    : iwindow(), aWx(wx), aWy(wy), aWw(ww), aWh(wh), aFx(fx), aFy(fy) {}
};

void percepts_window::remake(pvector& percepts)
{
  if (aExists)
    {
      // cout << "remaking percepts window\n";
      gui_window_delete(aWindow);
      aWindow= NULL;
      aExists= 0;

      create(percepts);
    }
}

void percepts_window::create(pvector& percepts)
{
  
  if (!aExists)
    {
      aN_percepts=  percepts.size();
      // cout << "creating Percepts window for " << aN_percepts << " percepts.\n";

      aCell_rects= new gui_rectangle[aN_percepts];
      aCell_labels= new gui_label[aN_percepts];

      aWindow= gui_window_new(name(), aWx, aWy, aWw, aWh);
      
      int rw= (int) floor((aWw - (2*aFx))/aN_percepts);
      int rh= aWh - (2 * aFy);

      for (uint i= 0; i<percepts.size(); i++)
	{
	  aCell_rects[i] = gui_rectangle_new(aWindow, aFx+(i*rw), aFy, rw, rh,  
					     GUI_YELLOW, true);
	  aCell_labels[i]= gui_label_new(aWindow, aFx + 10 + (i*rw), aFy + 25, 
					 percepts[i]->label()); 
	}
      gui_window_show(aWindow);
      aExists= 1;
    }
}

void percepts_window::update(pvector& percepts)
{
  const uint32 s_colors[]= {GUI_RED, GUI_GREEN, GUI_WHITE, GUI_BLUE, GUI_WHITE, 
			    GUI_RED, GUI_CYAN, GUI_BLACK, GUI_MAGENTA, GUI_RED };

  if (aExists)
    {

      for (uint i=0; i<percepts.size(); i++)
	{
	  // cout << "name: " << percepts[i]->name() << " value " << percepts[i]->value() << "\n";
	  if (percepts[i]->value()==1.0) 
	    gui_rectangle_change_color(aCell_rects[i], s_colors[i]);
	  else gui_rectangle_change_color(aCell_rects[i], GUI_YELLOW);
	  gui_label_show(aCell_labels[i]);
	}
    }
}

#endif





