/*****************************************************************************
 * File:         salience_window.h                                           *
 * Version:      4.0                                                         *
 * Description:  window class for displaying behavioural salience            *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef SALIENCE_WINDOW
#define SALIENCE_WINDOW

#include "iwindow.h"

#define MBEHAV 10;

class salience_window : public iwindow
{
 private:
  gui_rectangle aCell_rects[10][2];
  int           aWindow_clock;
  int           aN_behaviours;
  bool          aAllow_neg;
  int           aWx;
  int           aWy;
  int           aWw;
  int           aWh;
  int           aFw;
  int           aFh;
  int           aFx;
  int           aFy;
  int           aColw;                // width of a column for a single channel
  int           aHistw;               // width of a single histogram
  float         aMax_val;             // max y value ***
  int           aMax_histh;           // height of a single histogram ***
  int           aY_offset;

  gui_rectangle draw_sal(int, int, float, uint32);
  int get_h(float);                   // new function ***
 public:
  void create(bvector&);
  void remake(bvector&);
  void update(bvector&, float*, float*);        // update both columns simultaneously             
  // void update(float*, int);           // update one column at a time
  virtual char* name() { return "salience"; }
  salience_window(bool f= false, int wx= 0, int wy= 452, int ww= 410, int wh= 200, 
		  int fx= 30, int fy= 20) 
    : iwindow(), aAllow_neg(f), aWx(wx), aWy(wy), aWw(ww), aWh(wh), aFx(fx), aFy(fy), 
    aMax_val(1.1) {}
};

int salience_window::get_h(float v)
{
  return (int) ceil((v / aMax_val) * aMax_histh);
}

gui_rectangle salience_window::draw_sal(int col, int n, float value, uint32 color)
{
  int x, y, histh;
  
  if (value > aMax_val) value= aMax_val;
  else if (value < -aMax_val) value= -aMax_val;

  x= aFx + 1 + (col * aColw) + (n * aHistw);

  histh = get_h(value);   // ***

  if (histh >= 0) y= aY_offset - histh;
  else
    {
      y= aY_offset;
      histh= -histh;
    }

  return gui_rectangle_new(aWindow, x, y, aHistw, histh, color, true);
}

void salience_window::remake(bvector& behaviours)
{
  if (aExists)
    {
      cout << "remaking salience window\n";
      aExists= 0;
      gui_window_delete(aWindow);
      // delete[] aCell_rects;
      // aCell_rects= NULL;
      aWindow= NULL;

      cout << "a" << flush;
      create(behaviours);
      cout << "b\n";
    }
}

void salience_window::create(bvector& behaviours)
{

  int lab_x_offset, x_start;

  if (!aExists)
    {
      aN_behaviours= behaviours.size();
      // aCell_rects= new gui_rectangle[(aN_behaviours * 2) + 1];

      aY_offset= (aAllow_neg)? (int) (aWh/2.0) : (aWh - aFy);
      // cout << "aY_offset= " << aY_offset << endl;
      aFw= aWw - (2*aFx);
      aMax_histh= aWh - (2 * aFy + 1);
      aColw= (int) (aFw / aN_behaviours);
      aHistw= (int) ((aColw*0.8) / 2.0);

      aWindow= gui_window_new("salience", aWx, aWy, aWw, aWh);
      
      gui_line_new(aWindow, aFx, aY_offset, aWw - aFx, aY_offset, GUI_BLACK);
      gui_line_new(aWindow, aFx, aFy, aFx, aWh + 2 - aFy, GUI_BLACK);
      // axis ticks ***
      for (int i= 0; i<10; ++i)
	{
	  int y= aY_offset - get_h((1+i) * 0.1);
	  int w= ((i+1)%5==0)? 10 : 5; 
	  gui_line_new(aWindow, aFx, y, aFx - w, y, GUI_BLACK);
	  gui_line_new(aWindow, aFx, y + 1, aFx - w, y + 1, GUI_BLACK);
	}
      
      lab_x_offset= (int) (aColw / 3.0);
      x_start= aFx;

      for (uint i=0; i<behaviours.size(); i++)
	{
	  gui_label_new(aWindow, x_start + lab_x_offset, aFy, 
			behaviours[i]->label());
	  x_start += aColw;
	  aCell_rects[i][0]  = draw_sal(i, 0, 1.0, GUI_GREY);
	  aCell_rects[i][1]  = draw_sal(i, 1, 1.0, GUI_GREY);
	}
      gui_window_show(aWindow);
      aExists= 1;
      aWindow_clock= 0;
    }
}

/* update saliences window- two alternative methods */

void salience_window::update(bvector& behaviours, float *v1, float *v2)
{
  const bool use_behav_colors= true;
  // const int refresh= 200;

  int i;
  uint32 colors[]= { GUI_GREEN, GUI_YELLOW };
  
  if (aExists)
    {
      // if (aWindow_clock==refresh) remake(behaviours);
	
      for (i=0; i< aN_behaviours; i++)
	{
	  gui_rectangle_delete(aCell_rects[i][0]);
	  gui_rectangle_delete(aCell_rects[i][1]);
	  if (use_behav_colors)
	    {
	      uint32 c= get_behav_color(i);
	      aCell_rects[i][0]= draw_sal(i, 0, v1[i], c);
	      aCell_rects[i][1]= draw_sal(i, 1, v2[i], c);
	    }
	  else
	    {
	      aCell_rects[i][0]= draw_sal(i, 0, v1[i], colors[0]);
	      aCell_rects[i][1]= draw_sal(i, 1, v2[i], colors[1]);
	    }
	}
      aWindow_clock++;
    }
}

/*
void salience_window::update(float *v, int col)
{
  const uint32 colors[]= { GUI_GREEN, GUI_YELLOW };
  int i;
  
  if (col<0 || col>1) printf("salience window invalid column %d\n", col);
  else
    {
      if (aExists)
	{
	  for (i=0; i<aN_behaviours; i++)
	    {
	      gui_rectangle_delete(aCell_rects[2*i+col]);
	      draw_sal(i, col, v[i], colors[col]);
	    }
	}
    }
}
*/

#endif






