/*****************************************************************************
 * File:         percepts_window.h                                           *
 * Version:      4.0                                                         *
 * Description:  window class for displaying perceptual variables            *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef TIME_BAR
#define TIME_BAR

#include "iwindow.h"

class time_bar : public iwindow
{
 private:
  int           aWx;
  int           aWy;
  int           aWw;
  int           aWh;
  int           aFx;
  int           aFy;
  int           aFw;
  int           aFh;
 
 public:
  void create(int total_time);
  void update(double t);
  virtual char* name() { return "time"; }
  percepts_window(int wx= 420, int wy= 270, int ww= 575, int wh= 40, int fx= 20, int fy= 5) 
    : iwindow(), aWx(wx), aWy(wy), aWw(ww), aWh(wh), aFx(fx), aFy(fy) {}
};

void time_bar::create(int duration= 60)
{
  
  if (!aExists)
    {
      // cout << "creating time bar for duration " << duration << "\n";

      aFw= aWw - (2 * aFx);
      aFh= aWh - (2 * aFy);

      aWindow= gui_window_new(name(), aWx, aWy, aWw, aWh);
      gui_line_new(aWindow, aFx, aFy + aFh, aFx + aFw, aFy + aFh, GUI_BLACK);
      aWindow_frame= gui_rectangle_new(aWindow, aFx+1, aFy, aFw, aFh, GUI_GREY, true);
      gui_window_show(aWindow);
      aExists= 1;
    }
}

int last_x = 0;

void percepts_window::update(double t)
{
  const uint32 s_colors[]= {GUI_RED, GUI_GREEN, GUI_WHITE, GUI_BLUE, GUI_WHITE, 
			    GUI_RED, GUI_CYAN, GUI_BLACK, GUI_MAGENTA, GUI_RED };

  if (aExists)
    { 
      int x= (int) floor(t * aFw);
      while (x > last_x)
	{
	  gui_line_new(aWindow, x, aFy + aFh, x, aFy, GUI_RED);
	  ++ last_x;
	}
    }
}

#endif





