/*****************************************************************************
 * File:         percepts_window.h                                           *
 * Version:      4.0                                                         *
 * Description:  window class for displaying perceptual variables            *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef TIME_BAR
#define TIME_BAR

#include "iwindow.h"

class time_bar : public iwindow
{
 private:
  double        aMax_time;
  int           aWx;
  int           aWy;
  int           aWw;
  int           aWh;
  int           aFx;
  int           aFy;
  int           aFw;
  int           aFh;
  gui_rectangle aFull_frame;
  bool          aFull;

  int get_x(float t, int off= 0) { return off + (int) floor((t/aMax_time) * aFw); }
  void draw_dashes(float, int, int, uint32);
  void draw_bar();

 public:
  void create(bool, double);
  void remake(bool);
  void remake();
  void update(double, int);
  void countdown(float, float);
  double set_max_time(double t) 
    { 
      remake(); // was clear();
      // cout << "time_bar::set max time " << t << endl;
      aMax_time= t; draw_bar(); return t; 
    }
  virtual char* name() { return "time"; }
  time_bar() : iwindow() {}
};

void time_bar::draw_dashes(float inc, int offs, int thick= 2, uint32 col= GUI_BLACK)
{
  float t1= 0.0, t2;
  int y= aFy + aFh;
  bool stripe= true;

  while (t1< aMax_time)
    {
      uint32 c= (stripe)? col : GUI_WHITE;
      t2= t1 + inc;
      for (int i=0; i< thick; ++i)
	gui_line_new(aWindow, get_x(t1, aFx), y + offs + i, get_x(t2, aFx), y + offs + i, 
		     c);
      t1 += inc;
      stripe= !stripe;
    }
}

void time_bar::draw_bar()
{
  int y= aFy + aFh;

  gui_rectangle_show(aFull_frame);
  gui_line_new(aWindow, aFx, y, aFx + aFw, y, GUI_BLACK);
  // draw_dashes(2.0, 1, 3);
  draw_dashes(5.0, 1, 4);
  draw_dashes(30.0, 5, 4, GUI_BLUE);
}

void time_bar::countdown(float ct, float max_ct)
{
  int i= (int) ceil(max_ct - ct);
  int w= (int) floor(aFw / max_ct);

  if (i > 0)
    {
      int x= aFx + (i - 1) * w;
      uint32 col= (i == 1)? GUI_RED : GUI_BLUE;

      gui_rectangle_show(aFull_frame);
      gui_rectangle_new(aWindow, x, aFy, w, aFh, col, true);
    }
}

void time_bar::create(bool full= true, double mt= 240.0)
{
  
  if (!aExists)
    {
      aFull= full;
      aMax_time= mt;

      if (aFull)
	{
	  aWx= 420; aWy= 270;   // x, y position
	  aWw= 590; aWh= 40;    // width, height
	  aFx= 20;  aFy= 10;    // border width, height
	}
      else
	{
	  aWx= 105; aWy= 685;   // x, y position
	  aWw= 818; aWh= 60;    // width, height
	  aFx= 20;  aFy= 10;    // border width, height
	}
      
      aFw= aWw - (2 * aFx);
      aFh= aWh - (2 * aFy);

      // cout << "creating time bar for duration " << aMax_time << "(" << aFw << ")\n";
      aWindow= gui_window_new(name(), aWx, aWy, aWw, aWh);
      aFull_frame= gui_rectangle_new(aWindow, 0, 0, aWw, aWh, GUI_GREY, true);
      aWindow_frame= gui_rectangle_new(aWindow, aFx, aFy, aFw, aFh, GUI_GREY, true);
      gui_window_show(aWindow);
      aWindow_clock= 0;
      draw_bar();
      aExists= 1;
    }
}

void time_bar::remake(bool full)
{
  if (aExists)
    {
      // cout << "remaking time bar\n";
      gui_window_delete(aWindow);
      aWindow= NULL;
      aExists= 0;

      create(full);
    }
}

void time_bar::remake() { remake(aFull); }

void time_bar::update(double t, int w= -1)
{
  if (aExists)
    { 
      int x= get_x(t, 0);
      uint32 col= get_behav_color(w);

      // cout << "update time bar t=" << t << " max_t=" << aMax_time;
      // cout << " winner= " << w << endl;
      if (x >= aFw)
	{
	  x= x % aFw;
	  if (x==0) 
	    {
	      // cout << "Clear time bar\n";
	      remake();      // clear();
	    }
	}
      while (x > aWindow_clock)
	{
	  ++aWindow_clock;
	  int c= aWindow_clock + aFx;
	  gui_line_new(aWindow, c, aFy + aFh - 1, c, aFy, col);
	}
    }
}

#endif









