/*****************************************************************************
 * File:         tools_window.h                                              *
 * Version:      4.0                                                         *
 * Description:  window class for online control of model basal ganglia      *
 * Authors:      Tony Prescott and Fernando Montes                           *
 * Modified:     T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef TOOLS_WINDOW
#define TOOLS_WINDOW

#include <string>
#include <stdio.h>
#include "../ctrl/controller_gui.h"
#include "iwindow.h"

// global functions declaration used by Tools window
void gui_callback();

class tools_window : public iwindow
{
  gui_textfield aFolder_field;
  string aFolder_name;
  gui_button aActiveButton;
  vector <gui_button> aButtons;
  gui_popup aPopup;
 public:
  void set_button_enabled(int i, bool flag) {
    // cout << Name() << ": Set_button_enabled " << i << " " << (int) flag <<"\n";
    if (i < (int) aButtons.size()) {
      if (flag) gui_button_enable(aButtons[i]);
      else gui_button_disable(aButtons[i]);
    }
  }
  void set_button_label(int i, char label[]) {    // ***
    gui_button_change_text(aButtons[i], label);
  }
  string& efolder_name()
    {
      if (aExists) aFolder_name= gui_textfield_get_text(aFolder_field);
      return aFolder_name;
    }  
  void callback(flag_vector&);
  void create(string&, flag_vector&);
  virtual char* name() { return "tools"; }
  tools_window() : aFolder_name("") {}
};  


void tools_window::create(string& cname, flag_vector& flags)
{
  // cout << "creating tools window, exists= " << aExists << "\n";
  if (!aExists)
    {
      aWindow= gui_window_new(name(), 530, 360, 185, 150);
      const int bh= 25;
      const int n_da_values= 10;
 
      char* da_values_0[10]=
      { "0.06", "0.12", "0.20", "0.22", "0.23", "0.34", "0.37", "0.40", "0.43", "0.46"};     // da basic  
      char* da_values_1[10]= 
      { "0.0", "0.1", "0.2", "0.3", "0.325", "0.35", "0.4", "0.6", "0.8", "0.9" };       // da full
      
      char** da_values= (NEW_DA)? da_values_1 : da_values_0;

      gui_button g;
      for(unsigned i=0; i<flags.size(); i++)
	{
	  aButtons.push_back(g= gui_button_new(aWindow, 10, 5+(i*bh), flags[i]->label()));
	  gui_button_set_width(g, 155);
	  if (!flags[i]->enabled_at_start()) gui_button_disable(g);
	}
      
      float f= 10.0;                        // this bit gets the initial value
      int index= 0;
      for (int i= 0; i < n_da_values; ++i)
	{
	  sscanf(da_values[i], "%f", &f);
	  if (f<=global_da) index= i;
	}

      gui_label_new(aWindow, 20, 20 + (flags.size() * bh), "Dopamine");
      aPopup= gui_popup_new(aWindow, 90, 5 + (flags.size() * bh), 
			    n_da_values, da_values, index);

      char buf[80];
      my_c_str(buf, cname);
      aFolder_field= gui_textfield_new(aWindow, 10, 5+((1+ flags.size()) * bh), buf, 20);
      gui_textfield_disable(aFolder_field);

      gui_event_callback(gui_callback);    /* this gives a compiler warning !! */
      gui_window_show(aWindow);
      aExists= 1;
    }
}

void tools_window::callback(flag_vector& flags)
{
  if (gui_event_get_info()==GUI_WIDGET)
    {
      if (aPopup == gui_event_get_widget())
	{
	  float f;
	  sscanf(gui_popup_get_selected_text(aPopup), "%f", &f);
	  cout << "New Dopamine value: " << f << endl;
	  global_da= f;
	  aActiveButton = NULL;
	}
      else
	aActiveButton = gui_event_get_widget();
    }

  if (aActiveButton)
  {
    // cout << efolder_name() << "\n";
    gui_button_desactivate(aActiveButton);
    for (unsigned i= 0; i<aButtons.size(); i++)
      if (aActiveButton==aButtons[i])
	{
	  flags[i]->toggle();
	  gui_button_change_text(aButtons[i], flags[i]->label());
	}
    aActiveButton = NULL;
  }
}

#endif












