/*****************************************************************************
 * File:         bout.h                                                      *
 * Version:      1.0                                                         *
 * Description:  Object for storing record of behavioural data               *
 * Authors:      Tony Prescott                                               *
 * Contact:      t.j.prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef BOUT
#define BOUT

class bout
{
 private:
  int aId;
  char* aName;
  int aN;
  int aSteps;
  int aWait;

  double aStart;
  double aEnd;

  bool aComplete;
  stat aEfficiency;
  stat aDistortion;
 public:
  double duration() { return aEnd - aStart; }
  int steps() { return aSteps; }
  int id() { return aId; }
  int wait() { return aWait; }
  double finish() { aComplete= true; return duration(); }
  bool completed() { return aComplete; }
  virtual void update(double, float, float);
  void restart(double);
  virtual void clear();
  void print();
  bout(double, int, char*, int, int);
  virtual ~bout() { if (aName != NULL) delete[] aName; }
};


bout::bout(double et, int n, char* nm, int id, int wait) : 
  aId(id), aN(n), aSteps(0), aWait(wait), aStart(et), aEnd(et),
  aComplete(false), aEfficiency("efficiency"), aDistortion("distortion")
{
  // aName= NULL;
  aName= new char[strlen(nm)+1];
  strcpy(aName, nm);  
  // cout << "new bout " << nm << endl;
}

void bout::update(double et, float ef, float di) 
{ 
  aSteps+=1; 
  aEnd= et;
  aEfficiency.add_value(ef); 
  aDistortion.add_value(di); 
}

void bout::restart(double et) 
 { 
   aSteps= 0; aStart= aEnd= et; aComplete= false; 
 }

void bout::clear()
{ 
  restart(0.0);
  aEfficiency.zero(); aDistortion.zero(); 
}

void bout::print()
{
  cout << aN << '\t'  << aStart << '\t' << aEnd << '\t' << aName << '\t'; 
  cout << aId << '\t' << duration() << '\t' << aSteps << '\t' << aWait << '\t';
  cout << aEfficiency.mean() << '\t' << aEfficiency.median() << '\t';
  cout << aDistortion.mean() << '\t' << aDistortion.median();
  if (!aComplete) cout << " unfinished...";
  cout << endl;
}

// *** 27.01.03 defined but not used
// could be used by behaviour class to record activity over multiple bouts

class b_record : public bout
{
 private:
  int aTotal_steps;
  double aTotal_time;
  int aN_bouts;
 public:
  void finish() { bout::finish(); aTotal_steps+= steps(); aTotal_time+= duration(); aN_bouts+=1; }
  void clear() { bout::clear(); aTotal_steps=0; aTotal_time= 0.0;  aN_bouts= 0; }
  b_record(double et, char* nm) : bout(et, 0, nm, 0, 0) {}
  // virtual ~b_record() : ~bout() {}
};

#endif





















































