/*****************************************************************************
 * File:         utils/io.h                                                  *
 * Version:      4.0                                                         *
 * Description:  IO Utilities                                                *
 * Author:       Tony Prescott and Fernando Montes                           *
 * Contact:      T.J.Prescott@shef.ac.uk                                     *
 *****************************************************************************/

#ifndef UTILS_IO
#define UTILS_IO

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <stdio.h>

using namespace std;


// getting info from the user via terminal io

char* int_to_cstring(int i, char* buf)
{
  sprintf(buf, "%d", i);
  return buf;
}

string int_to_string(int i)
{
  char buf[32];
  sprintf(buf, "%d", i);
  string mystr(buf);
  return mystr;
}

string ftos(float f, int dp = 2)
{
  char buf[32];
  sprintf(buf, "%.*f", dp, f);
  string mystr(buf);
  return mystr;
}

char* my_c_str(char* buf, string& s)
{
  s.copy(buf, s.size());
  buf[s.size()]= 0;
  return buf;
}


void namecpy(char **to, char *from)
{
  if (*to != NULL) delete *to;
  if (from != NULL)
    {
      *to= new char[strlen(from)+1];
      strcpy(*to, from);
    }
  else *to= NULL;
}

void get_string(string& s, const string& prompt)
{
  while (s.empty()) {
    cout << prompt;
    getline(cin, s);
  }
}

string abb_cstring(char *cs, int n)
{
  string s(cs);
  return s.substr(0,n);
}

bool get_y_or_n(const string& prompt)
{
  char c;
  cout << prompt;
  cin >> c;
  cin.ignore(80, '\n'); // throw away the rest of the line

  // cout << "char is " << c <<".\n";
  if (c == 'y') return true;
  else if (c == 'n') return false;
  else return get_y_or_n(prompt);
}

int get_int(const string& prompt)
{
  int n;
  cout << prompt;
  cin >> n;
  cin.ignore(80, '\n'); // throw away rest of line
  return n;
}

float get_float(const string& prompt)
{
  float f;
  cout << prompt;
  cin >> f;
  cin.ignore(80, '\n'); // throw away rest of line
  return f;
}

// redirecting the standard output stream (cout)

streambuf* coutbuf= cout.rdbuf();  // store a pointer to the default cout buffer

bool redirect_cout(ofstream& outs, string fname, bool append= false, int verbose= 0)
{
  if (fname.length() < 3) {
    cerr << "redirect_cout: filename must have at least 3 characters '" << fname << "'\n";
    return false;
  }

  else if (append) outs.open(fname.c_str(), ios::out|ios::app);
  else outs.open(fname.c_str(), ios::out|ios::trunc);

  if (!outs) {
    cerr << "redirect_cout: can't open '" << fname << "' for output\n";
    return false;
  }
  else {
    if (verbose>0) cout << "writing data to '" << fname << "'\n";
    cout.rdbuf(outs.rdbuf()); // redirect cout
    return true;
  }
}

bool redirect_cout(ofstream& outs, string prompt, string suffix, bool append= false)
{
  string fname;
    
  get_string(fname, prompt);
  while (fname.length() < 3) {
    cerr << "file name must have at least 3 characters '" << fname << "'\n";
    get_string(fname, prompt);
  }
 
  if (!suffix.empty()) fname+= "." + suffix;
  return redirect_cout(outs, fname, append);
}

void reinstate_cout(ofstream& outs)
{
  outs.clear();            // clean-up and close current output stream
  outs.close();
  cout.rdbuf(coutbuf);     // redirect output to the screen
}


// formatting the standard output for tables of real numbers

ios::fmtflags cflags;
bool fmtflags_set= false;

void table_format_flags(int dp, bool show_pos= true)
{
  if (fmtflags_set) cerr << "warning: attempt to set format flags twice\n";
  else
    {
      fmtflags_set= true;
      cflags= cout.flags();  // store current flags
      cout.setf(ios::showbase);
      cout.setf(ios::fixed);
      if (show_pos) cout.setf(ios::showpos);
      cout.precision(dp);
    }
}

void reset_format_flags()
{
  cout.flags(cflags);
  fmtflags_set= false;
}

void table_format_flags(ofstream& out, int dp, bool show_pos= true)
{
  out.setf(ios::showbase);
  out.setf(ios::fixed);
  if (show_pos) out.setf(ios::showpos);
  out.precision(dp);
}

// file exists
// return a 1 if the file at fname exists or 0 if it doesnt

bool file_exists(string fname) 
{ 
	ifstream fs; 
    	fs.open(fname.c_str(), ios::in); // try opening the file 
    	if (!fs) { fs.close(); return false; } 
        else { fs.close(); return true; }
}


// create directory

/*
// unix
#include <sys/stat.h>
       #include <sys/types.h>
       #include <fcntl.h>
       #include <unistd.h>

int mkdir(const char *pathname, mode_t mode); 

if (mkdir("/dir", 0666) == -1 
*/

/*
// windows
#include <windows.h> 
CreateDirectory("C:\\games\\123",NULL); 
*/

// printing vector contents

template <class T> void print_vector(vector <T> v, int verbose) {
  for (unsigned i=0; i<v.size(); i++) v[i]->print(verbose);
}

template <class T> void pprint_vector(vector <T> v, char* starts= "", 
				      char* seps= ", ", char* ends= "\n") {
  cout << starts;
  for (unsigned i=0; i<v.size(); i++) 
    {
      v[i]->print(1); 
      if (i<(v.size()-1)) cout << seps;  
    }
  cout << ends;
}

template <class T> void ppprint_vector(vector <T> v, char* starts= "", 
				       char* seps= ", ", char* ends= "\n") {
  cout << starts;
  for (uint i=0; i<v.size(); i++) 
    {
      v[i].print(1); 
      if (i<(v.size()-1)) cout << seps;  
    }
  cout << ends;
}

void ppad(string& nm, int n, string ends= ": ")
{
  string s(nm);
  s.resize(n, ' ');
  cout << s << ends;
}
    
// printing various types to a filestream (this is Fernando's code)

void osprint(ostream& disp, float x, char end)
{
  disp << x << end;
}

void osprint(ostream& disp, double x, char end)
{
  disp << x << end;
}

void osprint(ostream& disp, char* s)
{
  disp << s;
}

void osprint(ostream& disp, int index, float array[], char end= '\n')
{
  int i;

  for (i=0;i<index;i++)
  {
   	disp << array[i];
    if (i+1<index) disp << '\t';
  }
  disp << end;
}

void osprint(ostream& disp, int index, int array[])
{
  int i;

  for (i=0;i<index;i++)
    disp << array[i] << " ";
  disp << "\n";
}

/*
void osprint(ostream& disp, int index, int32 array[])
{
  int i;

  for (i=0;i<index;i++)
    disp << array[i] << " ";
  disp << "\n";
}
*/


#endif
