/*****************************************************************************/
/* File:         timer.h                                                     */
/* Version:      2.0                                                         */
/* Description:  Timer utilities: timer and timer_stat classes               */
/* References:                                                               */
/* Author:       t.j.prescott@Sheffield.ac.uk                                */
/*                                                                           */
/*****************************************************************************/

#include <time.h>
#include <iostream.h>
#include <math.h>
#include <string>
#include <stdio.h>

string date_and_time_str()
{
  time_t *tptr = new time_t; 

  time(tptr);
  string s(ctime(tptr));
  return s;
}

string date_str()
{
  time_t tt= time(NULL);
  struct tm *t= localtime(&tt);
  char buf[32];
  sprintf(buf, "%d.%d.%d", t->tm_mday, t->tm_mon, t->tm_year%100);
  string mystr(buf);
  return mystr; 
} 

string time_str()
{
  time_t tt = time(NULL);
  struct tm *t= localtime(&tt);
  char buf[32];
  sprintf(buf, "%d:%d", t->tm_hour, t->tm_min);
  string mystr(buf);
  return mystr; 
} 

// timer class

class timer {
  char*  aName;
  int aScale;           // scaling factor (power 10) use 3 for milliseconds (default 1) 
  double aS;            // start time in seconds
  double aT;            // elapsed time between start and stop commands in seconds
  double get_seconds() { return (double) time(NULL); }
  double scale(double v) { return v*pow(10, aScale); }
 public:
  double read() { return  scale(aT); }
  double start() { aS= get_seconds(); aT= 0.0; return aT; }
  double stop() { aT= get_seconds() - aS; return scale(aT); }
  double check() { return scale(get_seconds() - aS); }
  void print(int verbose= 1) { 
    cout << aName << ": " << read();
    if (aScale==0) cout << "s";
    else if (aScale==3) cout << "ms";
    else cout << "s e-" << aScale;
    cout << endl;
  }
  timer(char* name= NULL, int scale= 0) : aName("timer"), aScale(scale), aT(0.0) 
    { if (name!=NULL) aName= name; }
};


// timer_stat: combines a timer with a stat
// saves a value everytime the clock is stopped


class timer_stat : public timer , public stat
{
 public:
  double stop() { return add_value(timer::stop()); }
  void print(int verbose= 1) { stat::print(); }
  timer_stat(char* nm, int scale= 1, bool save= false) : timer(nm, scale), stat(nm, save) {}
};












